using Remotable;

/**
*@METHOD		: Client.cs
*@DESCRIPTION	: Client chat.
*@VERSION		: NOVEMBER 1, 2015
*@PROGRAMMER	: BRIAN HINDS
*/

using System;
using System.Drawing;
using System.Media;
using System.Runtime.Remoting;
using System.Windows.Forms;

namespace Client
{
    public partial class ClientForm : Form
    {
        /**
       *@ Variable for user name, Host, and client configuration.
       */
        public string UserName = "";
        public string host = "http://localhost:8080/Chat";
        public string ClientConfig = "\\Client.exe.config";
        /**
        *@ Variable for mute button.
        */
        public int mute;
        public bool btnStat = true;
        /**
      *@ Variable for encryption.
      */
        public bool Encrpt = false;
        public string Encr = "";
        public string Decr = "";
        
        public ClientForm()
        {
            InitializeComponent();
            btnEncrpOn.Image = Image.FromFile(@"unlock.png");
            GetIP();
        }

        /**
       *@ Instantiating the Remote and Wrapper Class.
       */
        private RemoteClass connection;
        private WrapperClass chatMsg;
        private EncryptionService EService = new EncryptionService();
        /**
         *@METHOD			:GetIP()
        *@DESCRIPTION		:Sets the ip and userName for the messenger.
        *@PARAMETERS		:None
        *@RETURNS			:None
        */

        private void GetIP()
        {
            using (IPConfig IPSettings = new IPConfig())
            {
                if (IPSettings.ShowDialog() == DialogResult.OK)
                {
                    /**
                    *@ Check if an IP address have been entered.
                    */
                    if (!String.IsNullOrEmpty(IPSettings.TheIP))
                    {
                        /**
                        *@ Assigning server ip address.
                        */
                        host = "http://" + IPSettings.TheIP + ":8080/Chat";
                        /**
                        *@ Assigning UserName.
                        */
                        UserName = IPSettings.TheUserName;
                    }
                }
                else
                {
                    /**
                    *@ Assigning default local host address.
                    */
                    host = "http://localhost:8080/Chat";
                }
            }
        }

        /**
        *@METHOD			:ClientForm_Load
        *@DESCRIPTION		:Initialized the components of the form.
        *@PARAMETERS		:(object sender, EventArgs e)
        *@RETURNS			:None
        */

        private void ClientForm_Load(object sender, EventArgs e)
        {
            try
            {
                RemotingConfiguration.Configure(Application.StartupPath + ClientConfig, false);

                connection = (RemoteClass)Activator.GetObject(typeof(RemoteClass), host);
                chatMsg = new WrapperClass();
                connection.MessageReceived += new MessageHandler(chatMsg.WrapperMessageReceivedHandler);
                chatMsg.WrapperMessageReceived += new MessageHandler(ChatHandler);
            }
            catch (Exception)
            {
                MessageBox.Show("Unable to connect to remote server!", "No server found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                Application.Exit();
            }

            mute = 2;
        }

        /**
         *@METHOD			:buttonSend_Click
         *@DESCRIPTION		:This function executes the send command when the button is pressed..
         *@PARAMETERS		:(object sender, EventArgs e)
         *@RETURNS			:None
         */

        private void buttonSend_Click(object sender, EventArgs e)
        {
            /**
             *@ Checking if encryption is enabled.
             */
            if (Encrpt == true)
            {
                using (var KeyTone = new SoundPlayer(@"pop.wav"))
                {
                    /**
                     *@ Checking if the application have been mute.
                     */
                    if (mute == 2) { KeyTone.Play(); }

                    try
                    {
                        /**
                        *@ Checking for a user name before sending the message.
                        */
                        if (UserName == "")
                        /**
                         *@ Assigning a name if the user fails to assign one.
                         */
                        { UserName = "Anonymous"; }
                        /**
                         *@ Encrypting the message.
                         */
                        Encr = EService.Encrypt("[" + UserName + "]> " + composedMessage.Text, EService.key);
                        /**
                        *@ Sending encrypted message.
                        */
                        connection.Send(Encr);
                        /**
                        *@ Clearing composed window.
                        */
                        composedMessage.Clear();
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("No server connection detected, message was not sent.", "Server connection lost", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        Application.Exit();
                    }
                }
            }
            /**
            *@ Checking if encryption is disable.
            */
            else if (Encrpt == false)
            {
                using (var KeyTone = new SoundPlayer(@"pop.wav"))
                {
                    /**
                     *@ Checking if the application have been mute.
                     */
                    if (mute == 2) { KeyTone.Play(); }

                    try
                    {
                        /**
                        *@ Checking for a user name before sending the message.
                        */
                        if (UserName == "")
                        {
                            UserName = "Anonymous"; connection.Send("[" + UserName + "]" + "> " + composedMessage.Text);
                        }
                        /**
                        *@ Send the message with user assigned screen name.
                        */
                        else { connection.Send("[" + UserName + "]" + "> " + composedMessage.Text); }
                        composedMessage.Clear();
                    }
                    catch (Exception)
                    {
                        MessageBox.Show("No server connection detected, message was not sent.", "Server connection lost", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        Application.Exit();
                    }
                }
            }
        }

        /**
       *@METHOD			:ChatHandler
       *@DESCRIPTION	:This function handles the text and display it asynchronously.
       *@PARAMETERS		:(string newChatMsg)
       *@RETURNS		:None
       */

        public void ChatHandler(string newChatMsg)
        {
            /**
             *@ Searching for encryption enable command.
             */
            var check03 = newChatMsg.Contains("[Encryption Enable]");
            if (check03)
            {
                /**
                 *@ If command is found then it calls a method to auto enable encryption.
                 */
                AutoEncryption(newChatMsg);
                /**
                 *@ Encrypting message to be send to the decrypting method.
                 */
                newChatMsg = EService.Encrypt(newChatMsg, EService.key);
            }
            /**
             *@ If search comes be false then this syntax is executed.
             */
            if (check03 == false)
            {
                /**
                 *@ Searching the string for screen name start string.
                 */
                if (!newChatMsg.StartsWith("["))
                {
                    /**
                     *@ If string is not found, incoming message is encrypted
                     *@ and therefor will be decrypted.
                     */
                    var checkMsg = EService.Decrypt(newChatMsg, EService.key);
                    /**
                     *@ Checking for encryption disable command.
                     */
                    var check04 = checkMsg.Contains("[Encryption Disable]");
                    if (check04)
                    /**
                     *@ If the command is found then, the method to auto disable is called.
                     */
                    {
                        AutoEncryption(checkMsg);
                        /**
                        *@ Decrypt the encrypted message to be displayed.
                        */
                        newChatMsg = EService.Decrypt(newChatMsg, EService.key);
                    }
                }
            }
            /**
             *@ Check if encryption is enable.
             */
            if (Encrpt == true)
            {
                /**
                 *@ Decrypt the message to be displayed in the chat window.
                 */
                Decr = EService.Decrypt(newChatMsg, EService.key);

                using (var KeyTone = new SoundPlayer(@"receive01.wav"))
                {
                    /**
                     *@ Checking if the application have been muted.
                     */
                    if (mute == 2) { KeyTone.Play(); }
                    /**
                     *@ Checking if chat window have been invoked, if not the message is displayed.
                     */
                    if (MessageWindow.InvokeRequired == false)
                    { MessageWindow.AppendText("\n" + Decr + "\n"); }
                    else
                    {
                        var statusDelegate = new MessageHandler(crossThreading);
                        BeginInvoke(statusDelegate, new object[] { Decr });
                    }
                }
            }
            /**
            *@ Encryption Disable.
            */
            else if (Encrpt == false)
            {
                AutoEncryption(newChatMsg);

                using (var KeyTone = new SoundPlayer(@"receive01.wav"))
                {
                    if (mute == 2) { KeyTone.Play(); }

                    if (MessageWindow.InvokeRequired == false)
                    { MessageWindow.AppendText("\n" + newChatMsg + "\n"); }
                    else
                    {
                        var statusDelegate = new MessageHandler(crossThreading);
                        BeginInvoke(statusDelegate, new object[] { newChatMsg });
                    }
                }
            }
        }

        /**
        *@METHOD			:crossThreading
        *@DESCRIPTION		:This function handles cross threading and display the message in the window.
        *@PARAMETERS		:(string newChatMsg)
        *@RETURNS			:None
        */

        private void crossThreading(string newChatMsg)
        { MessageWindow.AppendText("\n" + newChatMsg + "\n"); }

        /**
        *@METHOD			:ClientForm_FormClosing
        *@DESCRIPTION		:This function handles what happens if the chat window is closed.
        *@PARAMETERS		:(object sender, FormClosingEventArgs e)
        *@RETURNS			:None
        */

        private void ClientForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"exit.wav"))
            {
                if (UserName == "") UserName = "Anonymous";

                KeyTone.Play();
                if (Encrpt == false)
                {
                    connection.Send("[" + UserName + "]" + "> Disconnected!");
                }
                else if (Encrpt == true)
                {
                    Encr = EService.Encrypt("[" + UserName + "]" + "> Disconnected!", EService.key); connection.Send(Encr);
                }

                try
                {
                    connection.MessageReceived -= new MessageHandler(chatMsg.WrapperMessageReceivedHandler);
                    chatMsg.WrapperMessageReceived -= new MessageHandler(ChatHandler);
                }
                catch (Exception ex)
                { MessageBox.Show(ex.Message); }
            }
        }

        /**
        *@METHOD			:toolStripButton1_Click
        *@DESCRIPTION		:This function handles what happens if the user logs out of chat.
        *@PARAMETERS		:(object sender, FormClosingEventArgs e)
        *@RETURNS			:None
        */

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"exit.wav"))
            {
                if (UserName == "") UserName = "Anonymous";

                if (MessageBox.Show("Do you want to log out and exit chat?", "Exit", MessageBoxButtons.YesNo, MessageBoxIcon.Information) == DialogResult.Yes)
                {
                    KeyTone.Play();

                    if (Encrpt == false)
                    {
                        connection.Send("[" + UserName + "]" + "> Has logged out!");
                    }
                    else if (Encrpt == true)
                    {
                        Encr = EService.Encrypt("[" + UserName + "]" + "> Has logged out!", EService.key); connection.Send(Encr);
                    }

                    try
                    {
                        connection.MessageReceived -= new MessageHandler(chatMsg.WrapperMessageReceivedHandler);
                        chatMsg.WrapperMessageReceived -= new MessageHandler(ChatHandler);
                    }
                    catch (Exception ex)
                    { MessageBox.Show(ex.Message); }

                    Application.Exit();
                }
            }
        }

        /**
        *@METHOD			:textcolor_Click
        *@DESCRIPTION		:This function change the text color of the chat window.
        *@PARAMETERS		:(object sender, EventArgs e)
        *@RETURNS			:None
        */

        private void textcolor_Click(object sender, EventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Click.wav"))
            {
                KeyTone.Play();
                using (var colorWheel = new ColorDialog())
                {
                    if (colorWheel.ShowDialog() == DialogResult.OK)
                    { MessageWindow.ForeColor = colorWheel.Color; KeyTone.Play(); }
                }
            }
        }

        /**
        *@METHOD			:msgFont_Click
        *@DESCRIPTION		:This function change the text font of the chat window.
        *@PARAMETERS		:(object sender, EventArgs e)
        *@RETURNS			:None
        */

        private void msgFont_Click(object sender, EventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Click.wav"))
            {
                KeyTone.Play();
                var result = fontDialog1.ShowDialog();

                if (result == DialogResult.OK)
                {
                    var font = fontDialog1.Font;

                    MessageWindow.Font = font;
                    KeyTone.Play();
                }
            }
        }

        /**
       *@METHOD			:composedMessage_KeyDown
       *@DESCRIPTION	:This function send a message when the enter key is pressed.
       *@PARAMETERS		:(object sender, KeyEventArgs e)
       *@RETURNS		:None
       */

        private void composedMessage_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                /**
                 *@ Encryption Enable.
                 */
                if (Encrpt == true)
                {
                    using (var KeyTone = new SoundPlayer(@"pop.wav"))
                    {
                        if (mute == 2) { KeyTone.Play(); }

                        try
                        {
                            if (UserName == "")
                            { UserName = "Anonymous"; }

                            Encr = EService.Encrypt("[" + UserName + "]> " + composedMessage.Text, EService.key);

                            connection.Send(Encr);

                            composedMessage.Clear();
                        }
                        catch (Exception)
                        {
                            MessageBox.Show("No server connection detected, message was not sent.", "Server connection lost", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            Application.Exit();
                        }
                    }
                }
                /**
                *@ Encryption Disable.
                */
                else if (Encrpt == false)
                {
                    using (var KeyTone = new SoundPlayer(@"pop.wav"))
                    {
                        if (mute == 2) { KeyTone.Play(); }

                        try
                        {
                            if (UserName == "")
                            {
                                UserName = "Anonymous"; connection.Send("[" + UserName + "]" + "> " + composedMessage.Text);
                            }
                            else { connection.Send("[" + UserName + "]" + "> " + composedMessage.Text); }
                            composedMessage.Clear();
                        }
                        catch (Exception)
                        {
                            MessageBox.Show("No server connection detected, message was not sent.", "Server connection lost", MessageBoxButtons.OK, MessageBoxIcon.Error);
                            Application.Exit();
                        }
                    }
                }
            }
        }

        /**
       *@METHOD			:about_Click
       *@DESCRIPTION	:This function calls the about dialog box when pressed..
       *@PARAMETERS		:(object sender, EventArgs e)
       *@RETURNS		:None
       */

        private void about_Click(object sender, EventArgs e)
        {
            using (var KeyTone = new SoundPlayer("Click.wav"))
            {
                KeyTone.Play();
                using (var about = new AboutChat())
                { about.ShowDialog(); }
            }
        }

        /**
       *@METHOD			:composedMessage_KeyPress
       *@DESCRIPTION	:This function plays a sound when a key is pressed
       *@PARAMETERS		:(object sender, KeyPressEventArgs e)
       *@RETURNS		:None
       */

        private void composedMessage_KeyPress(object sender, KeyPressEventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Keyboard.wav"))
            {
                if (mute == 2) { KeyTone.Play(); }
            }
        }

        /**
       *@METHOD			:UserWindow_KeyPress
       *@DESCRIPTION	:This function plays a sound when a key is pressed
       *@PARAMETERS		:(object sender, KeyPressEventArgs e)
       *@RETURNS		:None
       */

        private void UserWindow_KeyPress(object sender, KeyPressEventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Keyboard.wav"))
            {
                if (mute == 2) { KeyTone.Play(); }
            }
        }

        /**
       *@METHOD			:mutebtn_Click
       *@DESCRIPTION	:This function allows the user to mute messenger sounds.
       *@PARAMETERS		:(object sender, EventArgs e)
       *@RETURNS		:None
       */

        private void mutebtn_Click(object sender, EventArgs e)
        {
            if (btnStat == true)
            {
                mute = 1;
                btnStat = false;
                mutebtn.Image = Image.FromFile(@"Mute.png");
            }
            else if (btnStat == false)
            {
                mute = 2;
                btnStat = true;
                mutebtn.Image = Image.FromFile(@"Speaker.png");
            }
        }

        /**
        *@METHOD			:
        *@DESCRIPTION		:
        *@PARAMETERS		:
        *@RETURNS			:
        */

        private void btnEncrpOn_Click(object sender, EventArgs e)
        {
            if (Encrpt == false)
            {
                Encrpt = true;

                btnEncrpOn.Image = Image.FromFile(@"lock.png");

                connection.Send("[Encryption Enable]");
            }
            else if (Encrpt == true)
            {
                Encrpt = false;
                btnEncrpOn.Image = Image.FromFile(@"unlock.png");

                Encr = EService.Encrypt("[Encryption Disable]", EService.key);

                connection.Send(Encr);
            }
        }

        /**
        *@METHOD			:
        *@DESCRIPTION		:
        *@PARAMETERS		:
        *@RETURNS			:
        */

        public void AutoEncryption(string encrypVarify)
        {
            var check = encrypVarify.Contains("[Encryption Enable]");
            if (check)
            {
                Encrpt = true;

                btnEncrpOn.Image = Image.FromFile(@"lock.png");
            }
            var check02 = encrypVarify.Contains("[Encryption Disable]");
            if (check02)
            {
                Encrpt = false;

                btnEncrpOn.Image = Image.FromFile(@"unlock.png");
            }
        }
    }
}