﻿/**
*@METHOD		: 
*@DESCRIPTION	: 
*@VERSION		: 
*@PROGRAMMER	: 
*/

using System;
using System.IO;
using System.Linq;
using System.Security.Cryptography;
using System.Windows.Forms;

namespace Client
{
    public class EncryptionService
    {
        /*128C Encryption Key*/
        public string key = "8RQS16buYQcQwnHNQ95rMSay9nhxkJ4McJSp4Ctz1q5mHrb3dTaaCZ8miKN8JQjZHZHLRNXgj5bymChw3FKQWweHBDkoTGWfRch1jeep31DAm1JdlsFBlgodkFHBN55c";
        /*Encryption Size*/
        private readonly int EncryptionSize = 64;
        /**
        *@METHOD			:Encrypt
        *@DESCRIPTION		:This method encrypt the message to be sent
        *@PARAMETERS		:string OriginalMsg, string key
        *@RETURNS			:WriteMemoryStream(OriginalMsg, ref saltBytes, encryptor, ref memoryStream, ref cryptoStream);
        */

        public string Encrypt(string OriginalMsg, string key)
        {
            /*Check for the composed message thats to be encrypted*/
            if (string.IsNullOrEmpty(OriginalMsg))
            {
                MessageBox.Show("Encryption Service did not find any message to encrypt", "Encryption Error",MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            /*Check for the internal encryption key*/
            if (string.IsNullOrEmpty(key))
            {
                throw new ArgumentNullException("Internal encryption key error\nIncorrect Key Size for chosen encryption.");
            }

            using (var keyDerivationFunction = new Rfc2898DeriveBytes(key, EncryptionSize))
            {
                var saltBytes = keyDerivationFunction.Salt;
                var keyBytes = keyDerivationFunction.GetBytes(32);
                var ivBytes = keyDerivationFunction.GetBytes(16);

                using (var aesManaged = new AesManaged())
                {
                    aesManaged.KeySize = 256;

                    using (var encryptor = aesManaged.CreateEncryptor(keyBytes, ivBytes))
                    {
                        MemoryStream memoryStream = null;
                        CryptoStream cryptoStream = null;

                        /*Returns the encryption results*/
                        return WriteMemoryStream(OriginalMsg, ref saltBytes, encryptor, ref memoryStream, ref cryptoStream);
                    }
                }
            }
        }

        /**
        *@METHOD			:
        *@DESCRIPTION		:
        *@PARAMETERS		:
        *@RETURNS			:
        */

        public string Decrypt(string EncryptedMsg, string key)
        {
            if (string.IsNullOrEmpty(EncryptedMsg))
            {
                throw new ArgumentNullException("EncryptedMsg");
            }

            if (string.IsNullOrEmpty(key))
            {
                throw new ArgumentNullException("Internal encryption key error\nIncorrect Key Size for chosen encryption.");
            }


            var allTheBytes = Convert.FromBase64String(EncryptedMsg);
            var saltBytes = allTheBytes.Take(EncryptionSize).ToArray();
            var EncryptedMsgBytes = allTheBytes.Skip(EncryptionSize).Take(allTheBytes.Length - EncryptionSize).ToArray();

            using (var keyDerivationFunction = new Rfc2898DeriveBytes(key, saltBytes))
            {
                var keyBytes = keyDerivationFunction.GetBytes(32);
                var ivBytes = keyDerivationFunction.GetBytes(16);

                return DecryptWithAES(EncryptedMsgBytes, keyBytes, ivBytes);
            }
        }

        /**
        *@METHOD			:
        *@DESCRIPTION		:
        *@PARAMETERS		:
        *@RETURNS			:
        */

        private string WriteMemoryStream(string OriginalMsg, ref byte[] saltBytes, ICryptoTransform encryptor, ref MemoryStream memoryStream, ref CryptoStream cryptoStream)
        {
            try
            {
                memoryStream = new MemoryStream();

                try
                {
                    cryptoStream = new CryptoStream(memoryStream, encryptor, CryptoStreamMode.Write);

                    using (var streamWriter = new StreamWriter(cryptoStream))
                    {
                        streamWriter.Write(OriginalMsg);
                    }
                }
                finally
                {
                    if (cryptoStream != null)
                    {
                        cryptoStream.Dispose();
                    }
                }

                var EncryptedMsgBytes = memoryStream.ToArray();
                Array.Resize(ref saltBytes, saltBytes.Length + EncryptedMsgBytes.Length);
                Array.Copy(EncryptedMsgBytes, 0, saltBytes, EncryptionSize, EncryptedMsgBytes.Length);

                return Convert.ToBase64String(saltBytes);
            }
            finally
            {
                if (memoryStream != null)
                {
                    memoryStream.Dispose();
                }
            }
        }

        /**
        *@METHOD			:
        *@DESCRIPTION		:
        *@PARAMETERS		:
        *@RETURNS			:
        */

        private static string DecryptWithAES(byte[] EncryptedMsgBytes, byte[] keyBytes, byte[] ivBytes)
        {
            using (var aesManaged = new AesManaged())
            {
                using (var decryptor = aesManaged.CreateDecryptor(keyBytes, ivBytes))
                {
                    MemoryStream memoryStream = null;
                    CryptoStream cryptoStream = null;
                    StreamReader streamReader = null;

                    try
                    {
                        memoryStream = new MemoryStream(EncryptedMsgBytes);
                        cryptoStream = new CryptoStream(memoryStream, decryptor, CryptoStreamMode.Read);
                        streamReader = new StreamReader(cryptoStream);

                        return streamReader.ReadToEnd();
                    }
                    finally
                    {
                        if (memoryStream != null)
                        {
                            memoryStream.Dispose();
                            memoryStream = null;
                        }
                    }
                }
            }
        }
    }
}