/**
*@METHOD		: Client.cs
*@DESCRIPTION	: Client chat.
*@VERSION		: NOVEMBER 1, 2015
*@PROGRAMMER	: BRIAN HINDS
*/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.Remoting;
using Remotable;
using Server;
using System.Runtime.Remoting.Messaging;
using System.Media;

namespace Client
{
    public partial class Interceptor : Form
    {
        /**
       *@ Variable for username, Host, and client configuration.
       */
        public string UserName = "";
        public string host = "http://localhost:8080/Chat";
        public string ClientConfig = "\\Client.exe.config";
        /**
       *@ Variable for mute button.
       */
        public int mute;
        public bool btnStat = true;
        public Interceptor()
        {
            InitializeComponent();
        }
        /**
       *@ Instantiating the Remote and Wrapper Class.
       */
        RemoteClass connection;
        WrapperClass chatMsg;
        
        /**
        *@METHOD			:ClientForm_Load
        *@DESCRIPTION		:Initialized the components of the form.
        *@PARAMETERS		:(object sender, EventArgs e)
        *@RETURNS			:None
        */
        private void ClientForm_Load(object sender, EventArgs e)
        {
            try {

                RemotingConfiguration.Configure(Application.StartupPath + ClientConfig, false);
                connection = (RemoteClass)Activator.GetObject(typeof(RemoteClass), host);
                chatMsg = new WrapperClass();
                connection.MessageReceived += new MessageHandler(chatMsg.WrapperMessageReceivedHandler);
                chatMsg.WrapperMessageReceived += new MessageHandler(ChatHandler);
            }
            catch(Exception)
            {
                MessageBox.Show("Unable to connect to remote server!");
            }

           mute = 2;
        }
      
        /**
       *@METHOD			:ChatHandler
       *@DESCRIPTION	:This function handles the text and dis play it asynchronously.
       *@PARAMETERS		:(string newChatMsg)
       *@RETURNS		:None
       */
        public void ChatHandler(string newChatMsg)
        {
            using (var KeyTone = new SoundPlayer(@"receive.wav"))
            {
                if (mute == 2) { KeyTone.Play(); }
                
                if (testwindow.InvokeRequired == false)
                { testwindow.AppendText("\n"+newChatMsg + "\n"); }
                else
                {
                    var statusDelegate = new MessageHandler(crossThreading);
                    BeginInvoke(statusDelegate, new object[] { newChatMsg });
                }
            }
        }
         /**
         *@METHOD			:crossThreading
         *@DESCRIPTION		:This function handles cross threading and display the message in the window.
         *@PARAMETERS		:(string newChatMsg)
         *@RETURNS			:None
         */
        private void crossThreading(string newChatMsg)
        { testwindow.AppendText("\n" + newChatMsg + "\n"); }
        /**
        *@METHOD			:ClientForm_FormClosing
        *@DESCRIPTION		:This function handles what happens if the chat window is closed.
        *@PARAMETERS		:(object sender, FormClosingEventArgs e)
        *@RETURNS			:None
        */
        private void ClientForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"exit.wav"))
            {
                              
                try
                {
                    connection.MessageReceived -= new MessageHandler(chatMsg.WrapperMessageReceivedHandler);
                    chatMsg.WrapperMessageReceived -= new MessageHandler(ChatHandler);
                }
                catch (Exception ex)
                { MessageBox.Show(ex.Message); }
            }

        }
       
        /**
        *@METHOD			:textcolor_Click
        *@DESCRIPTION		:This function change the text color of the chat window.
        *@PARAMETERS		:(object sender, EventArgs e)
        *@RETURNS			:None
        */
        private void textcolor_Click(object sender, EventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Click.wav"))
            {
                KeyTone.Play();
                using (var colorWheel = new ColorDialog())
                {
                    if (colorWheel.ShowDialog() == DialogResult.OK)
                    { testwindow.ForeColor = colorWheel.Color; KeyTone.Play(); }
                }
            }
        }
        /**
        *@METHOD			:msgFont_Click
        *@DESCRIPTION		:This function change the text font of the chat window.
        *@PARAMETERS		:(object sender, EventArgs e)
        *@RETURNS			:None
        */
        private void msgFont_Click(object sender, EventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Click.wav"))
            {
                KeyTone.Play();
                var result = fontDialog1.ShowDialog();

                if (result == DialogResult.OK)
                {
                    var font = fontDialog1.Font;
                   
                    testwindow.Font = font;
                    KeyTone.Play();
                }
            }
        }
       
        /**
       *@METHOD			:about_Click
       *@DESCRIPTION	:This function calls the about dialog box when pressed..
       *@PARAMETERS		:(object sender, EventArgs e)
       *@RETURNS		:None
       */
        private void about_Click(object sender, EventArgs e)
        {
            using (var KeyTone = new SoundPlayer("Click.wav"))
            {
                KeyTone.Play();
                using (var about = new AboutChat())
                { about.ShowDialog(); }
            }
        }
        /**
       *@METHOD			:composedMessage_KeyPress
       *@DESCRIPTION	:This function plays a sound when a key is pressed
       *@PARAMETERS		:(object sender, KeyPressEventArgs e)
       *@RETURNS		:None
       */
        private void composedMessage_KeyPress(object sender, KeyPressEventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Keyboard.wav"))
            {
                if (mute == 2){KeyTone.Play(); }
            }
        }
        /**
       *@METHOD			:UserWindow_KeyPress
       *@DESCRIPTION	:This function plays a sound when a key is pressed
       *@PARAMETERS		:(object sender, KeyPressEventArgs e)
       *@RETURNS		:None
       */
        private void UserWindow_KeyPress(object sender, KeyPressEventArgs e)
        {
            using (var KeyTone = new SoundPlayer(@"Keyboard.wav"))
            {
                if(mute == 2){KeyTone.Play();}
            }
        }
      
       

       
    }

}