/**
*@METHOD		: RemotableClass.cs
*@DESCRIPTION	: This class handles what happens during the
*@              : send and receiving of a message.
*@VERSION		: NOVEMBER 1, 2015
*@PROGRAMMER	: BRIAN HINDS
*/
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Remoting;
using System.Runtime.InteropServices;
using System.Runtime.Remoting.Messaging;


namespace Remotable
{
    /**
    *@ This method will handle the received event.
    */
    [Serializable]
    [ComVisible(true)]
    public delegate void MessageHandler(string chatMsg);
    /**
    *@METHOD			:RemoteClass : MarshalByRefObject
    *@DESCRIPTION		:This method handles the chat task when a chat is received.
    *@PARAMETERS		:None
    *@RETURNS			:None
    */
    public class RemoteClass : MarshalByRefObject
    {
        public event MessageHandler MessageReceived;
        private MessageHandler messageDelegate;
        
        /**
       *@METHOD			:RemoteClass()
       *@DESCRIPTION	:This method initializes a new instance of the remote class.
       *@PARAMETERS		:None
       *@RETURNS		:None
       */
        public RemoteClass()
        {messageDelegate = new MessageHandler(Send);}
        /**
        *@METHOD		:InitializeLifetimeService()
        *@DESCRIPTION	:This method determine the life time of an object.
        *@PARAMETERS	:None
        *@RETURNS		:Null
        */
        public override object InitializeLifetimeService()
        {return null;}

        private delegate void WrapperDelegate(string chatMsg, MessageHandler messageDelegate);
        /**
         *@METHOD		:Send
         *@DESCRIPTION	:This method allows chat to be broadcast.
         *@PARAMETERS	:(string chatMsg)
         *@RETURNS		:None
         */
        public void Send(string chatMsg)
        {
            if (MessageReceived != null)
            {
                MessageHandler messageDelegate = null;
                Delegate[] invocationList_ = null;
                try
                {invocationList_ = MessageReceived.GetInvocationList();}
                catch (MemberAccessException ex)
                {throw new Exception("ERROR", ex);}
                if (invocationList_ != null)
                {
                    lock (this)
                    {foreach (Delegate del in invocationList_){
                            messageDelegate = (MessageHandler)del;
                            var chatDelegate = new WrapperDelegate(BeginSend);
                            var callback = new AsyncCallback(EndSend);
                            chatDelegate.BeginInvoke(chatMsg, messageDelegate, callback, chatDelegate);}}}}
        }
        /**
         *@METHOD		:BeginSend
         *@DESCRIPTION	:This method handles the time the chat takes to be broadcast.
         *@PARAMETERS	:(string chatMsg, MessageHandler messageDelegate)
         *@RETURNS		:None
         */
        private void BeginSend(string chatMsg, MessageHandler messageDelegate)
        {
            try
            {System.Threading.Thread.Sleep(100); messageDelegate?.Invoke(chatMsg);}
            catch (Exception){MessageReceived -= messageDelegate;}
        }
        /**
         *@METHOD		:EndSend
         *@DESCRIPTION	:This method handles async results of the chat data sent.
         *@PARAMETERS	:(IAsyncResult result)
         *@RETURNS		:None
         */
        private static void EndSend(IAsyncResult result)
        {
            var chatDelegate = (WrapperDelegate)result.AsyncState; chatDelegate.EndInvoke(result);
        }
      }
    }
